package com.adc.da.util.utils;

import com.google.common.collect.ArrayListMultimap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Enumeration;

/**
 * Py4j字典类<br/>
 * 该类定义了字典文件的存放路径、文件名、文件中的分隔符等，同时负责读取字典文件内容并初始化到集合，以供Py4jUtils类使用<br/>
 * 
 * @author zhangchao
 */
public class Py4jDictionary {

	private final Logger logger = LoggerFactory.getLogger(this.getClass());

	// 字典文件存放路径，默认为：src/java/resources/py4j目录
	private static final String PREFIX = "py4j/";
	// 多音字集合
	private ArrayListMultimap<String, String> duoYinZiMap;
	// 字典文件名，默认为：py4j.dic
	private static final String CONFIG_NAME = "py4j.dic";
	// 字典文件中拼音和汉字的分隔符，如：a#阿 
	private static final String PINYIN_SEPARATOR = "#";
	// 字典文件中汉字的分隔符，如：bang#膀/磅/蚌
	private static final String WORD_SEPARATOR = "/";
	// 是否初始化
	private volatile boolean initialized;

	/**
	 * 默认构造器
	 */
	public Py4jDictionary() {

	}

	/**
	 * 获取Py4jDictionary默认实例
	 * @return
	 */
	public static Py4jDictionary getDefault() {
		return SingletonHolder.INSTANCE;
	}

	/**
	 * 获取多音字集合<br/>
	 * 加载时先检查是否初始化
	 * @return
	 */
	public ArrayListMultimap<String, String> getDuoYinZiMap() {
		checkInit();
		return duoYinZiMap;
	}

	/**
	 * 检查多音字集合是否初始化，如果没有加载多音字集合
	 */
	private void checkInit() {
		if (!initialized) {
			loadVocabulary();
		}
	}

	/**
	 * 加载多音字集合
	 */
	private synchronized void loadVocabulary() {
		if (initialized) {
			return;
		}
		this.duoYinZiMap = loadVocabulary0(CONFIG_NAME);
		initialized = true;
	}

	/**
	 * 加载多音字集合
	 */
	private ArrayListMultimap<String, String> loadVocabulary0(String name) {
		debug("******start load py4j config******");
		ArrayListMultimap<String, String> duoYinZiMap = ArrayListMultimap.create(512, 32);
		// 配置文件路径
		String filename = PREFIX + name;
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			Enumeration<URL> urls = cl.getResources(filename);
			if (urls != null) {
				while (urls.hasMoreElements()) {
					URL url = urls.nextElement();
					parseURL(url, duoYinZiMap);
				}
			}
		} catch (Exception e) {
			error("caught exception when load py4j vocabulary", e);
			throw new RuntimeException("caught exception when load py4j vocabulary", e);
		}
		debug("******load py4j config over******");
		debug("py4j map key size:{}", duoYinZiMap.keySet().size());
		return duoYinZiMap;
	}

	/**
	 * 解析py4j多音字配置文件<br/>
	 * 多音字配置文件的内容如下：ao#拗口/违拗/拗断/执拗/拗口/拗口风/拗口令/拗曲/拗性/拗折/警拗<br/>
	 * 解析时，先按#拆分成数组，数组的第一个元素是拼音，第二个元素是多音字，<br/>
	 * 再将数组的第二个元素按/拆分成数组，<br/>
	 * 最后将所有的多音字放入到MAP集合中，key值是拼音，value值是多音字
	 */
	private void parseURL(URL url, ArrayListMultimap<String, String> duoYinZiMap) {
		debug("load py4j dictionary file:{}", url.getPath());
		InputStream in = null;
		BufferedReader br = null;
		try {
			in = url.openStream();
			br = new BufferedReader(new InputStreamReader(in, "UTF-8"));
			String line = null;
			while ((line = br.readLine()) != null) {

				String[] arr = line.split(PINYIN_SEPARATOR);

				if (StringUtils.isNotEmpty(arr[1])) {
					String[] dyzs = arr[1].split(WORD_SEPARATOR);
					for (String dyz : dyzs) {
						if (StringUtils.isNotEmpty(dyz)) {
							duoYinZiMap.put(arr[0], dyz.trim());
						}
					}
				}
			}
		} catch (IOException e) {
			throw new RuntimeException(String.format("load py4j config:%s error", url), e);
		} finally {
			IOUtils.closeQuietly(br);
			IOUtils.closeQuietly(in);
		}
	}

	private void error(String msg, Throwable err) {
		logger.error(msg, err);
	}

	private void debug(String msg, Object... args) {
		if (logger.isDebugEnabled()) {
			logger.debug(msg, args);
		}
	}

	private static class SingletonHolder {
		private static final Py4jDictionary INSTANCE = new Py4jDictionary();
	}
}
