package com.adc.da.util.utils;

import java.util.Arrays;
import java.util.HashSet;

import com.adc.da.util.exception.AdcDaBaseException;
import com.google.common.collect.ArrayListMultimap;

import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;

/**
 * 汉字转拼音工具类
 * 
 * @author zhangchao
 *
 */
public class Py4jUtils {

	// 多音字集合
	private static final ArrayListMultimap<String, String> duoYinZiMap = Py4jDictionary.getDefault().getDuoYinZiMap();

	/**
	 * 获取单个汉字的拼音，如果字符不是汉字，并且ASCII码位33-125，直接返回。ASCII码表：http://www.asciitable.
	 * com/
	 * 
	 * @param ch
	 *            汉字字符
	 * @return 拼音字符串
	 * @throws AdcDaBaseException
	 */
	public static String[] getPinyin(char ch) throws AdcDaBaseException {
		try {
			// 拼音格式：小写、无音标、 'ü'转为"v"，如chang
			HanyuPinyinOutputFormat outputFormat = new HanyuPinyinOutputFormat();
			outputFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);// 无音标
			outputFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);// 小写
			outputFormat.setVCharType(HanyuPinyinVCharType.WITH_V);// 'ü'转为"v"
			// ASCII >=33 ASCII<=125的直接返回,ASCII码表：http://www.asciitable.com/
			if (ch >= 32 && ch <= 125) {
				return new String[] { String.valueOf(ch) };
			}
			return distinct(PinyinHelper.toHanyuPinyinStringArray(ch, outputFormat));
		} catch (BadHanyuPinyinOutputFormatCombination e) {
			throw new AdcDaBaseException();
		}

	}

	/**
	 * 获取汉语拼音，如果字符不是汉字，并且ASCII码位33-125，直接返回字符。ASCII码表：http://www.asciitable.com/
	 * 
	 * @param chinese
	 *            汉字字符串
	 * @return 汉语拼音字符串，首字母大写，如长沙返回ChangSha；如果参数为空，返回null
	 * @throws AdcDaBaseException
	 */
	public static String getPinyin(String chinese) throws AdcDaBaseException {
		// 如果参数为空，返回null
		if (StringUtils.isEmpty(chinese)) {
			return null;
		}
		// 过滤掉一些特殊符号
		chinese = chinese.replaceAll("[\\.，\\,！·\\!？\\?；\\;\\(\\)（）\\[\\]\\:： ]+", " ").trim();

		StringBuilder py_sb = new StringBuilder(32);
		// 将字符串转换为字符数组
		char[] chs = chinese.toCharArray();
		// 遍历字符数组，获取汉字对应的拼音，如果是多音字，查询多音字字典
		for (int i = 0; i < chs.length; i++) {
			// 获取汉字对应的拼音
			String[] py_arr = getPinyin(chs[i]);
			// 如果没有获取到拼音，抛出异常
			if (py_arr == null || py_arr.length < 1) {
				throw new AdcDaBaseException("pinyin array is empty, char:" + chs[i] + ",chinese:" + chinese);
			}
			// 根据数组长度判断是否是多音字，并进行不同的处理
			if (py_arr.length == 1) {
				// 如果汉字不是多音字
				py_sb.append(convertInitialToUpperCase(py_arr[0]));
			} else if (py_arr.length == 2 && py_arr[0].equals(py_arr[1])) {
				// 如果汉字是多音字，并且多个拼音拼音相同，但是声调不同
				py_sb.append(convertInitialToUpperCase(py_arr[0]));
			} else {
				// 如果汉字是多音字，并且多个拼音不同，按如下规则取拼音
				// 1、向左多取一个字,例如 银[行],如果在多音字字典里有，则返回
				// 2、向右多取一个字,例如 [长]沙,如果在多音字字典里有，则返回
				// 3、左右各多取一个字,例如 龙[爪]槐,如果在多音字字典里有，则返回
				// 4、向左多取2个字,如 芈月[传],列车[长],如果在多音字字典里有，则返回
				// 5、向右多取2个字,如 [长]孙无忌,如果在多音字字典里有，则返回
				// 6、以上都不满足,取多音字字典里的默认拼音,如果还没有,取第一个拼音
				String resultPy = null, defaultPy = null;
				for (String py : py_arr) {
					String left = null; // 向左多取一个字,例如 银[行]
					if (i >= 1 && i + 1 <= chinese.length()) {
						left = chinese.substring(i - 1, i + 1);
						if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(left)) {
							resultPy = py;
							break;
						}
					}

					String right = null; // 向右多取一个字,例如 [长]沙
					if (i <= chinese.length() - 2) {
						right = chinese.substring(i, i + 2);
						if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(right)) {
							resultPy = py;
							break;
						}
					}

					String middle = null; // 左右各多取一个字,例如 龙[爪]槐
					if (i >= 1 && i + 2 <= chinese.length()) {
						middle = chinese.substring(i - 1, i + 2);
						if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(middle)) {
							resultPy = py;
							break;
						}
					}
					String left3 = null; // 向左多取2个字,如 芈月[传],列车长
					if (i >= 2 && i + 1 <= chinese.length()) {
						left3 = chinese.substring(i - 2, i + 1);
						if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(left3)) {
							resultPy = py;
							break;
						}
					}

					String right3 = null; // 向右多取2个字,如 [长]孙无忌
					if (i <= chinese.length() - 3) {
						right3 = chinese.substring(i, i + 3);
						if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(right3)) {
							resultPy = py;
							break;
						}
					}

					if (duoYinZiMap.containsKey(py) && duoYinZiMap.get(py).contains(String.valueOf(chs[i]))) { // 默认拼音
						defaultPy = py;
					}
				}

				if (StringUtils.isEmpty(resultPy)) {
					if (StringUtils.isNotEmpty(defaultPy)) {
						resultPy = defaultPy;
					} else {
						resultPy = py_arr[0];
					}
				}
				py_sb.append(convertInitialToUpperCase(resultPy));
			}
		}

		return py_sb.toString();
	}

	/**
	 * 转换首字母拼音为大写，如长:Chang
	 * 
	 * @param str
	 * @return
	 */
	private static String convertInitialToUpperCase(String str) {
		if (str == null || str.length() == 0) {
			return "";
		}
		return str.substring(0, 1).toUpperCase() + str.substring(1);
	}

	private static String[] distinct(String[] arr) {
		if (arr == null || arr.length == 0) {
			return arr;
		}
		HashSet<String> set = new HashSet<>();
		for (String str : arr) {
			set.add(str);
		}
		return set.toArray(new String[0]);
	}
	
	public static void main(String[] args) {
		String[] pinyinArray = Py4jUtils.getPinyin('长');
		System.out.println(Arrays.toString(pinyinArray));
		
		String pinyinString = Py4jUtils.getPinyin("我要买长安福特轿车");
		System.out.println(pinyinString);
		
		char[] chs = {'长', '行', '藏', '度', '阿', '佛', '2', 'A', 'a'};
		for(char ch : chs){
		    String[] arr_py = Py4jUtils.getPinyin(ch);
		    System.out.println(ch+"\t"+Arrays.toString(arr_py));
		}
	}

}
